/*!
	@brief	MIDI parsing library header

	@file	midi.h
	@author	Satoshi "Chuck" Takagi
	@date	2006/12/09
 */
/* 
	$RCSfile: midi.c,v $
	$Revision: 28 $
	$Date: 2006-12-09 18:03:58 +0900 (土, 09 12 2006) $
 */

#ifndef MIDI_H
#define MIDI_H

/*! MIDI ChannelVoice message */
#define MIDI_NOTE_OFF 0x80
#define MIDI_NOTE_ON  0x90
#define MIDI_P_PLS    0xA0
#define MIDI_C_CHG    0xB0
#define MIDI_P_CHG    0xC0
#define MIDI_C_PLS    0xD0
#define MIDI_PBEND    0xE0

/*! MIDI System message */
#define MIDI_MIDITC   0xF1
#define MIDI_SONG_PTR 0xF2
#define MIDI_SONG_SEL 0xF3
#define MIDI_TUNE_REQ 0xF6

#define ALL_NOTE_OFF  0x7B
#define RESET_ALL_CONTROLLERS 0x79

#define MIDI_CC_LEGATO 0x44


//extern uint8_t rb;
extern uint8_t g_midi_ch;
//extern uint8_t midi_recv_ch;
//extern uint8_t midi_byte[2];

//! MIDI_SystemMessage構造体
struct MIDI_SystemMessage {
	//! Status
	/*! 記録されているシステムメッセージの種類を指し示すタグ */
	uint16_t Status;
	//! union
	/*! 引数を伴うシステムメッセージの内容を記録する共用体 */
	union {
		uint8_t Timecode;				//!< タイムコード
		uint16_t SongPositionPointer;	//!< ソングポジション
		uint8_t SongSelect;				//!< ソングセレクト
	};
} MidiSysMes;

//! MIDI_ChannelMessage構造体
/*! */
struct MIDI_ChannelMessage {
	//! Status
	/*! 記録されているチャンネルメッセージの種類を指し示すタグ */
	uint8_t Status;
	//! チャンネルメッセージの内容をあらわす共用体
	/*! */
	union {
		//! Note On
		struct {
			uint8_t Note;		//!< Note On Note
			uint8_t Velocity;	//!< Note On Velocity
		} NoteOn;
		//! Note Off
		struct {
			uint8_t Note;
			uint8_t Velocity;
		} NoteOff;
		//! Polyphonic Pressure
		struct {
			uint8_t Note;
			uint8_t Value;
		} PolyPressure;
		//! Control Change
		struct {
			uint8_t Control;
			uint8_t Value;
		} ControlChange;
		//! Program Change
		struct {
			uint8_t Value;
		} ProgramChange;
		//! Channel Pressure
		struct {
			uint8_t Value;
		} ChannelPressure;
		//! Pitch Bend
		struct {
			uint16_t Value;
		} PitchBend;
		//! Mode Change
		struct {
			uint8_t Mode;
			uint8_t Value;
		} ModeChange;
	};
} MidiChanMes;


//extern uint8_t MIDI_CommonMessage_status;
extern uint8_t MIDI_RealtimeMessage_status;
//extern uint8_t MIDI_ChannelMessage_status;

//! MIDI_ChannelMessage構造体用 Statusフラグ
#define fMidiNoteOn				(0x01)
#define fMidiNoteOff			(0x02)
#define fMidiPolyPressure		(0x04)
#define fMidiControlChange		(0x08)
#define fMidiProgramChange		(0x10)
#define fMidiChannelPressure	(0x20)
#define fMidiPitchBend			(0x40)
#define fMidiModeChange			(0x80)

//! MIDI_SystemMessage構造体用 Statusフラグ
#define fMidiStartOfExclusive		(0x0001)
#define fMidiQuaterFrame			(0x0002)
#define fMidiSongPositionPointer	(0x0004)
#define fMidiSongSelect				(0x0008)
// undefined						(0x0010)
// undefined						(0x0020)
#define fMidiTuneRequest			(0x0040)
#define fMidiEndOfExclusive			(0x0080)
#define fMidiTimingClock			(0x0100)
// undefined						(0x0200)
#define fMidiStart					(0x0400)
#define fMidiContinue				(0x0800)
#define fMidiStop					(0x1000)
// undefined						(0x2000)
#define fMidiActiveSensing			(0x4000)
#define fMidiSystemReset			(0x8000)



/* prototypes */
extern uint8_t MidiSystemMessage(void);
extern void MidiChannelMessage(void);



#endif 
